;; testing the CMUCL XREF functionality
;;
;;
;; to use, (setf c::*record-xref-info* t) the compile this file. You
;; can then examine the various types of cross-references that have
;; been identified by saying
;;
;;  (maphash (lambda (key value) (format t "~S <-~{ ~S~}~%" key value)) c::*who-calls*)
;;  (maphash (lambda (key value) (format t "~S <-~{ ~S~}~%" key value)) c::*who-uses*)
;;  (maphash (lambda (key value) (format t "~S <-~{ ~S~}~%" key value)) c::*who-sets*)
;;  (maphash (lambda (key value) (format t "~S <-~{ ~S~}~%" key value)) c::*who-binds*)
;;
;; To reset the XREF database, use (c::init-xref-database). 


(in-package :cl-user)


(defvar *global-variable-one* 66642)

(defvar *global-variable-two* 66643)

(defconstant +global-constant-one+ 66644)


(defmacro macro-one (macro-argument)
  `(* 3 *global-variable-one* ,macro-argument))

(defun global-function-one (arg)
  (format t "Hi there mate ~d"
          (apply #'* *global-variable-one* *global-variable-two* (list arg)))
  (terpri)
  (setf *global-variable-two* 666656)
  ;; another use of a global variable
  (force-output *standard-output*))

;; should detect a call to the functions FORMAT, TERPRI, PRINC,
;; GLOBAL-FUNCTION-ONE. Should detect a setting of the
;; *GLOBAL-VARIABLE-ONE*. Should detect a binding of the
;; *GLOBAL-VARIABLE-ONE*. Should detect a use of the
;; *GLOBAL-VARIABLE-ONE*. Note that the call to 1+ is not detected,
;; because the compiler inlines FLET-FUNCTION-ONE, and by constant
;; propagation eliminates the call to 1+.
(defun global-function-two ()
  (flet ((flet-function-one (one) (1+ one)))
    (format t "~d~%" (global-function-one (flet-function-one 45)))
    (let ((*global-variable-one* 44445))
      (terpri)
      (incf *global-variable-one* 66699)
      (princ *global-variable-one*))))

;; should detect a use of MACRO-ONE, and a use of the global variable
;; *GLOBAL-VARIABLE-ONE*. Should also detect a use of the constant
;; +GLOBAL-CONSTANT-ONE+.
(defun global-function-three (foo)
  (declare (type fixnum foo))
  (macro-one (+ foo +global-constant-one+)))


;; example from ACL documentation
(defvar *var1* 3)
(+ *var1* 3)
(defmacro addit (x) `(+ *var1* ,x))
(defun calladdit (x y) (expt (addit x) y))
(defun call2 (a b) (calladdit a b))


(defclass foobase ()
  ((base :initform 42 :reader foo-base)))

(defclass fooclass (foobase)
   ((name :initarg :name :reader foo-name)
    (barg :initarg :barg :accessor get-foo-barg)))

(defgeneric zorble (x))

(defgeneric blarf (x))

(defmethod blarf ((x t))
    (car x))

(defmethod blarf ((x fooclass))
    (get-foo-barg x)
    (foo-name x)
    (setf *var1* 3)
    *var1*)

(defmethod zorble ((x t)) nil)

(defmethod zorble ((x fooclass))
  (+ +global-constant-one+ (blarf x)))


;; EOF
